import 'package:_private_core_network/options.dart';
import 'package:dio/browser.dart';
import 'package:dio/dio.dart';
import 'package:pretty_dio_logger/pretty_dio_logger.dart';

class AppClient extends DioForBrowser {
  static AppClient? _instance;
  static bool _enableErrorHandler = true;

  factory AppClient({
    required bool requiredToken,
    bool isAuthorizationCustom = false,
    String? token,
    String? baseUrl,
    bool enableErrorHandler = true,
    BaseOptions? options,
    List<Interceptor>? customInterceptors,
    Duration? connectTimeout,
    Duration? receiveTimeout,
    bool httpShouldUsePipelining = false,
  }) {
    _enableErrorHandler = enableErrorHandler;

    _instance ??= AppClient._(
      baseUrl: baseUrl ?? appBaseUrl,
      options: options,
      customInterceptors: customInterceptors,
      connectTimeout: connectTimeout,
      receiveTimeout: receiveTimeout,
    );
    if (options != null) _instance!.options = options;
    _instance!.options.baseUrl = baseUrl ?? appBaseUrl;
    (_instance!.transformer as BackgroundTransformer).jsonDecodeCallback =
        parseJson;
    if ((token == null || token.isEmpty)) {
      _instance!.options.headers.remove(r'Authorization');
    } else if (requiredToken) {
      _instance!.options.headers.addAll({
        r'Authorization': isAuthorizationCustom ? token : ('Bearer $token')
      });
    }
    _instance!.options.headers.addAll(appHeaders);
    return _instance!;
  }

  AppClient._({
    String? baseUrl,
    BaseOptions? options,
    List<Interceptor>? customInterceptors,
    Duration? connectTimeout,
    Duration? receiveTimeout,
  }) : super(options) {
    interceptors.add(InterceptorsWrapper(
      onRequest: (_, __) =>
          _requestInterceptor(_, __, connectTimeout, receiveTimeout),
      onResponse: _responseInterceptor,
      onError: _errorInterceptor,
    ));
    if (customInterceptors?.isNotEmpty == true) {
      interceptors.addAll(customInterceptors!);
    }
    if (networkOptions.loggingEnable) {
      interceptors.add(
        PrettyDioLogger(
          requestHeader: networkOptions.loggingrequestHeader,
          requestBody: networkOptions.loggingrequestBody,
          responseBody: networkOptions.loggingrequestBody,
          responseHeader: networkOptions.loggingrequestHeader,
          error: networkOptions.loggingerror,
          compact: networkOptions.loggingcompact,
          maxWidth: networkOptions.loggingmaxWidth,
        ),
      );
    }
    this.options.baseUrl = baseUrl ?? appBaseUrl;
  }

  _requestInterceptor(
    RequestOptions ops,
    RequestInterceptorHandler handler,
    Duration? connectTimeout,
    Duration? receiveTimeout,
  ) async {
    switch (ops.method) {
      case methodGet:
        break;
      case methodPost:
      case methodPut:
      case methodDelete:
        if (ops.baseUrl.contains('api.arckipel')) {
          if (ops.data is FormData) {
            ops.data = platformUuidFormData(ops.data);
          } else if (ops.data is Map) {
            ops.data = platformUuidParms(ops.data);
          }
        }
      default:
        break;
    }
    if (ops.baseUrl.contains('api.arckipel')) {
      ops.queryParameters = appMapParms(ops.queryParameters);
    }
    ops.connectTimeout = connectTimeout ?? const Duration(seconds: 30000);
    ops.receiveTimeout = receiveTimeout ?? const Duration(seconds: 30000);
    handler.next(ops);
  }

  _responseInterceptor(Response r, ResponseInterceptorHandler handler) {
    handler.next(r);
  }

  _errorInterceptor(DioException e, ErrorInterceptorHandler handler) {
    if (_enableErrorHandler) {
      if (errorInterceptor != null) errorInterceptor!(e);
    }
    handler.next(e);
  }
}
